#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_SET_LED    0
#define RQ_GET_SWITCH 1

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t light_var[2];
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
	
    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_val = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];
        
        if (led_val)
        {
            PORTD |= (1<<PD0)|(1<<PD1);
			play(20);
        }
        else
        {   
            PORTD &= ~(1<<PD0)&~(1<<PD1);
			pause();
        }
        return 0;
    }

    else if (rq->bRequest == RQ_GET_SWITCH)
    {
        if ((PINC & (1<<PC3)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        /* point usbMsgPtr to the data to be returned to host */
        usbMsgPtr = (uchar*) &switch_state;

        /* return the number of bytes of data to be returned to host */
        return sizeof(switch_state);
    }

    
    return 0;
}

void play(uint16_t freq)
{
    TCCR0B = 0b00001100;
    OCR0A = (((16000000L/256)/2)/freq)-1;
}

void pause()
{
    TCCR0B = 0b00000000;
}


/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRD  = (1<<PD0)|(1<<PD1)|(1<<PD6);
    DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); ;
    PORTC = (1 << PC3);
    TCCR0A = 0b01000010;
    TCCR0B = 0b00000100;
    usbInit();
    
    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();
    
    /* enable global interrupts */
    sei();
    /* main event loop */
    for(;;)
    {
        usbPoll();
        _delay_ms(1);
    }
    
    return 0;
}

/* ------------------------------------------------------------------------- */
